import * as THREE from "three";
import { EventEmitter } from "events";
import Camera from "./Camera";
import Renderer from "./Renderer";
import Sizes from "./utils/Sizes";
import Time from "./utils/Time";
import Resources from "./utils/Resources";
import Debug from "./utils/Debug";
import { disposeMaterial } from "./utils/helpers";
import { events } from "./static";
export class QuickThreejs extends EventEmitter {
    static instance;
    static tickEvent;
    static resizeEvent;
    scene;
    camera;
    renderer;
    sizes;
    time;
    resources;
    debug;
    canvas;
    /**
     * @param props {@link InitThreeProps}
     * @param appDom The app Dom element reference
     */
    constructor(props, appDom = "canvas#app") {
        super();
        if (QuickThreejs.instance)
            return QuickThreejs.instance;
        QuickThreejs.instance = this;
        // SETUP
        this.scene = new THREE.Scene();
        this.sizes = new Sizes({
            height: props?.sceneSizes?.height,
            width: props?.sceneSizes?.width,
            listenResize: props?.autoSceneResize
        });
        this.time = new Time();
        this.canvas =
            document.querySelector(appDom) ??
                document.createElement("canvas");
        this.camera = new Camera(props?.camera, !!props?.withMiniCamera);
        this.resources = new Resources(props?.sources);
        this.debug = new Debug(props?.enableDebug);
        this.renderer = new Renderer();
        if (typeof props?.axesSizes === "number") {
            const AXES_HELPER = new THREE.AxesHelper(props?.axesSizes);
            this.scene.add(AXES_HELPER);
        }
        if (typeof props?.gridSizes === "number") {
            const GRID_HELPER = new THREE.GridHelper(props?.gridSizes, props?.gridSizes);
            this.scene.add(GRID_HELPER);
        }
        QuickThreejs.tickEvent = () => this.update();
        QuickThreejs.resizeEvent = () => this.resize();
        this.time.on(events.TICKED, QuickThreejs.tickEvent);
        this.sizes.on(events.RESIZED, QuickThreejs.resizeEvent);
        this.emit(events.CONSTRUCTED);
    }
    resize() {
        this.camera.resize();
        this.renderer.resize();
    }
    update() {
        this.debug?.stats?.begin();
        this.emit(events.BEFORE_UPDATE);
        this.camera.update();
        this.debug?.update();
        this.emit(events.PRE_UPDATED);
        this.renderer.update();
        this.emit(events.UPDATED);
        this.debug?.stats?.end();
    }
    destruct() {
        this.time.destruct();
        this.sizes.destruct();
        this.camera.destruct();
        this.renderer.destruct();
        this.debug?.destruct();
        this.resources.destruct();
        this.scene.traverse((object) => {
            if (object instanceof THREE.Mesh) {
                object.geometry.dispose();
                if (Array.isArray(object.material)) {
                    for (let index = 0; index < object.material.length; index++) {
                        const material = object.material[index];
                        disposeMaterial(material);
                    }
                }
                else {
                    disposeMaterial(object.material);
                }
            }
            else if (object instanceof THREE.Light)
                object.dispose();
        });
        this.scene.userData = {};
        QuickThreejs.instance = undefined;
        this.emit(events.DESTRUCTED);
        this.removeAllListeners();
    }
}
//# sourceMappingURL=index.js.map